@@ -62,8 +62,8 @@ pda_pp <- function(cl, lambda=0.2) {
     stop("You need to select the class variable!")
   if (length(unique(cl)) < 2)
     stop("PDA index needs at least two classes!")
-  
-  # Convert to class to sequential integers, and sort
+
+  # Convert class to sequential integers, and sort
   cl.i <- as.integer(as.factor(cl))
   cl.sort <- order(cl.i)  
   cl <- cl.i[cl.sort]
@@ -72,8 +72,11 @@ pda_pp <- function(cl, lambda=0.2) {
     mat <- as.matrix(mat)
 
     # Reorder so classes are adjacent
-    mat <- mat[cl.sort, ]
-    
+    if (ncol(mat) == 1)
+      mat <- as.matrix(mat[cl.sort, ])
+    else
+      mat <- mat[cl.sort, ]
+
     ngroup <- table(cl.i)     # the obs. no. in each class, stored in table	  
     groups <- length(ngroup)  # no. of classes
     gname <- names(ngroup)    # names of classes, now is integer 1,2,3...
@@ -93,66 +96,73 @@ pda_pp <- function(cl, lambda=0.2) {
 # @param lambda shrinkage parameter (0 = no shrinkage, 1 = full shrinkage)
 .CalIndex<-function(n, p, groups, fvals, groupraw, gname, ngroup, lambda) { 
   # int i, j, k, right, left
-  g = groups
-  mean = matrix(rep(0,g*p),g,p)
-  ovmean = matrix(rep(0,p),p)
-  group = matrix(rep(0,n),n)  # the class label of each obs
+  g <- groups
+  mean <- matrix(rep(0, g*p), g, p)
+  ovmean <- matrix(rep(0, p), p)
+  group <- matrix(rep(0, n), n)  # the class label of each obs
 
-  right = n-1
-  left = 0
+  right <- n-1
+  left <- 0
  
-  group = groupraw;
+  group <- groupraw;
   
-  val = 0
+  val <- 0
 
 # Calculate mean for within class and the overall mean
   for (i in 1:n) {
     for (j in 1:p) {
-      mean[group[i],j] = mean[group[i],j] + fvals[i,j]/ngroup[group[i]]
-      ovmean[j] = ovmean[j] + fvals[i,j]/n
+      mean[group[i],j] <- mean[group[i],j] + fvals[i,j]/ngroup[group[i]]
+      ovmean[j] <- ovmean[j] + fvals[i,j]/n
     }
   }
 
-  cov = matrix(rep(0,p*p),p,p)
-  tempcov = matrix(rep(0,p*p),p,p)
+  cov <- matrix(rep(0,p*p),p,p)
+  tempcov <- matrix(rep(0,p*p),p,p)
 
   for (i in 1:n) {
     for (j in 1:p) {
       for (k in 1:j) {
-        cov[k,j] = cov[k,j] + (1-lambda)*(fvals[i,j] -
+        cov[k,j] <- cov[k,j] + (1-lambda)*(fvals[i,j] -
              mean[group[i],j]) * (fvals[i,k]-mean[group[i],k])
-        cov[j,k] = cov[k,j]
+        cov[j,k] <- cov[k,j]
       }
     }
   }
   
   for (j in 1:p)
-    cov[j,j] = cov[j,j] + lambda*n
+    cov[j,j] <- cov[j,j] + lambda*n
 
-  tempcov = cov
+  tempcov <- cov
 
 # =========================================================
-  pivot = matrix(rep(0,p),p)
-  det = 0
+  if (ncol(tempcov) > 1) {
+    pivot <- matrix(rep(0, p), p)
+    det <- 0
 
-  val = .ludcomp(tempcov, p, pivot)
+    val <- .ludcomp(tempcov, p, pivot)
+  }
+  else
+    val <- as.numeric(tempcov)
 # ===========================================================
 
   for (j in 1:p)
     for (k in 1:p)
       for (i in 1:g)
-	cov[j,k] = cov[j,k] + (1-lambda) * ngroup[i] *
+	cov[j,k] <- cov[j,k] + (1-lambda) * ngroup[i] *
           (mean[i,j] - ovmean[j]) * (mean[i,k] - ovmean[k])
 
-  tempcov = cov
-  tempval = .ludcomp(tempcov, p, pivot)
+  tempcov <- cov
+  if (ncol(tempcov) > 1) 
+    tempval <- .ludcomp(tempcov, p, pivot)
+  else
+    tempval <- tempcov
   
   if (tempval < 0.00000001) {
-    val = 0
+    val <- 0
     print ("ZERO VARIANCE!")
   }
   else
-    val = 1 - val/tempval
+    val <- 1 - val/tempval
   
   return(val)
 }
@@ -160,53 +170,53 @@ pda_pp <- function(cl, lambda=0.2) {
 #==============================================================================
 .ludcomp <- function(a, n, pivot)
 {
-  det = 1;
-  temp = 0;
-  s = matrix(rep(0,n),n)
+  det <- 1;
+  temp <- 0;
+  s <- matrix(rep(0, n), n)
   for (i in 1:n) {
-    s[i]=a[i,1+1]
+    s[i] <- a[i,1+1]
     for (j in 2:n)
-      if (s[i]<a[i,j]) 
-        s[i] = a[i,j]
+      if (s[i] < a[i,j]) 
+        s[i] <- a[i,j]
   }
-  
+
   for( k in 1:(n-1)) {
     for ( i in k:n )
     {
-      temp = abs(a[i,k]/s[i])
+      temp <- abs(a[i,k]/s[i])
       if (i==k) {
-        c = temp
-        pivot[k] = i
+        c <- temp
+        pivot[k] <- i
       }
       else if (c<temp)  {
-	c = temp
-	pivot[k] = i
+	c <- temp
+	pivot[k] <- i
       }
     }
 
     if (pivot[k] != k)
     {
-      det = det * (-1)
+      det <- det * (-1)
       for (j in k:n) {
-        temp = a[k,j]
-        a[k,j] = a[pivot[k],j]
-        a[pivot[k],j] = temp
+        temp <- a[k,j]
+        a[k,j] <- a[pivot[k],j]
+        a[pivot[k],j] <- temp
       }
-      temp = s[k]
-      s[k] = s[pivot[k]]
-      s[pivot[k]] = temp
+      temp <- s[k]
+      s[k] <- s[pivot[k]]
+      s[pivot[k]] <- temp
     }
-	
+
     for ( i in (k+1):n) {
-      temp = a[i,k]/a[k,k]
-      a[i,k] = temp
+      temp <- a[i,k]/a[k,k]
+      a[i,k] <- temp
       for (j in (k+1):n)
-        a[i,j] = a[i,j] - temp*a[k,j]
+        a[i,j] <- a[i,j] - temp*a[k,j]
       }
-      det = det * a[k,k]
+      det <- det * a[k,k]
   }
-  
-  det = det * a[n,n]
+
+  det <- det * a[n,n]
   
   return(det)
 
